"""interactive session"""
import os
from argparse import ArgumentParser
from fireworks.fw_config import LAUNCHPAD_LOC
from fireworks.fw_config import CONFIG_FILE_DIR
from fireworks.scripts.lpad_run import get_lp
from virtmat.language.interpreter.session_manager import SessionManager

ROOT_DIR = os.path.dirname(os.path.abspath(__file__))
GRAMMAR_PATH = os.path.join(ROOT_DIR, '..', 'grammar', 'virtmat.tx')


def parse_clargs():
    """parse the command line arguments for the script"""
    m_description = 'Execute a program remotely in deferred mode'
    parser = ArgumentParser(description=m_description)
    parser.add_argument('-c', '--config_dir',
                        help='path to configuration file (if -l unspecified)',
                        default=CONFIG_FILE_DIR)
    parser.add_argument('-l', '--launchpad_file',
                        help='path to launchpad file', default=LAUNCHPAD_LOC)
    parser.add_argument('-g', '--grammar_path', required=False,
                        help='path to grammar root file', default=GRAMMAR_PATH)
    parser.add_argument('-r', '--autorun', default=True, required=False,
                        action='store_true', help='run the model')
    parser.add_argument('-u', '--uuid', default=None, required=False,
                        help='UUID of a model')
    parser.add_argument('-d', '--on-demand', default=False, required=False,
                        action='store_true', help='run model nodes on demand')
    parser.add_argument('-a', '--async-run', default=False, required=False,
                        action='store_true', help='run model in the background')
    return parser.parse_args()


def main():
    """main function"""
    clargs = parse_clargs()
    mgr = SessionManager(get_lp(clargs), uuid=clargs.uuid, autorun=clargs.autorun,
                         grammar_path=clargs.grammar_path, on_demand=clargs.on_demand,
                         async_run=clargs.async_run)
    mgr.main_loop()


if __name__ == '__main__':
    main()
