"""apply constraints for parallelizable objects"""
from virtmat.language.utilities.textx import is_reference
from textx.exceptions import TextXError
from textx import get_children_of_type, get_parent_of_type, get_location
from textx import textx_isinstance


def check_parallelizable_processor(model, metamodel):
    """
    Apply constraints to parallelizable (Map, Filter and Reduce) objects and
    add attributes to their parent Variable objects
    """
    for cls in ['Map', 'Filter', 'Reduce']:  # classes with nchunks attribute
        for obj in get_children_of_type(cls, model):
            if obj.nchunks is not None:
                etype = 'Parallelization error'
                parent = get_parent_of_type('Variable', obj)
                if parent is None or parent.parameter is not obj:
                    msg = 'Parallel map, filter and reduce must be variable parameters'
                    raise TextXError(msg, **get_location(obj), err_type=etype)
                if textx_isinstance(obj, metamodel['Map']):
                    if not all(is_reference(p, metamodel) for p in obj.params):
                        msg = 'All parallel map parameters must be references'
                        raise TextXError(msg, **get_location(obj), err_type=etype)
                else:  # Filter or Reduce
                    if not is_reference(obj.parameter, metamodel):
                        msg = 'Parallel filter/reduce parameter must be a reference'
                        raise TextXError(msg, **get_location(obj), err_type=etype)
                setattr(parent, 'nchunks', obj.nchunks)
