"""
Register model processors

Model processors are callables that are called at the end of the parsing when
the whole model is instantiated. These processors accept the model and metamodel
as parameters.

Do not register object processors here
"""
from .cyclic import check_cycles_processor
from .duplicates import check_duplicates_processor
from .functions import check_functions_processor
from .imports import check_imports_processor
from .typechecks import check_types_processor
from .parallel import check_parallelizable_processor


def add_constraints_processors(metamodel):
    """register the constraints processors on the metamodel instance"""
    metamodel.register_model_processor(check_duplicates_processor)
    metamodel.register_model_processor(check_cycles_processor)
    metamodel.register_model_processor(check_imports_processor)
    metamodel.register_model_processor(check_functions_processor)
    metamodel.register_model_processor(check_types_processor)
    metamodel.register_model_processor(check_parallelizable_processor)
