# pylint: disable=protected-access
"""units properties for some metamodel classes"""
from virtmat.language.utilities.textx import isinstance_m


def number_units(self):
    """evaluate the units of a number"""
    if not hasattr(self, '__units'):
        self.__units = self.value.units
    return self.__units


def variable_units(self):
    """evaluate variable units"""
    if not hasattr(self, '__units'):
        if isinstance_m(self.parent, ['VarTuple']):
            vars_ = self.parent.variables
            units = self.parent.parameter.units  # fixme
            self.__units = next(t for var, t in zip(vars_, units) if var == self)
        else:
            if hasattr(self.parameter, 'units'):
                self.__units = self.parameter.units
            else:
                self.__units = None
    return self.__type


def reference_units(self):
    """evaluate the units of a referenced object"""
    if not hasattr(self, '__units'):
        self.__units = self.ref.units
    return self.__units


def add_units_properties(metamodel):
    """Add object class properties using monkey style patching"""
    metamodel['Number'].units = property(number_units)
    metamodel['Variable'].units = property(variable_units)
    metamodel['VarReference'].units = property(reference_units)
