"""helper functions to interface fireworks"""
from fireworks import FWorker
from fireworks.core.rocket_launcher import rapidfire, launch_rocket


def run_fireworks(lpad, fw_ids, create_subdirs=False, log_lvl='INFO'):
    """launch fireworks with the provided fw_ids"""
    worker = FWorker(query={'fw_id': {'$in': fw_ids}})
    if create_subdirs:
        rapidfire(lpad, fworker=worker, nlaunches=len(fw_ids), strm_lvl=log_lvl)
    for _ in fw_ids:
        launch_rocket(lpad, fworker=worker, strm_lvl=log_lvl)


def get_nodes_providing(lpad, fw_ids, name):
    """from the list of fw_ids find the nodes providing a specified output"""
    fw_query = {'fw_id': {'$in': fw_ids}, 'spec._tasks.outputs': name}
    return lpad.get_fw_ids(fw_query)


def get_parent_nodes(lpad, fw_ids, firework):
    """from the list of fw_ids find parent nodes for a new firework"""
    if all('inputs' in t and len(t['inputs']) == 0 for t in firework.tasks):
        # node with no inputs is a child of the root node
        parents = lpad.get_fw_ids({'fw_id': {'$in': fw_ids}, 'name': '_fw_root_node'})
        assert len(parents) == 1
    else:
        inputs = [i for t in firework.tasks for i in t['inputs']]
        parents = []
        for inp in inputs:
            nodes = get_nodes_providing(lpad, fw_ids, inp)
            if len(nodes) != 0:
                assert len(nodes) == 1
                parents.append(nodes[0])
            else:
                parents.append(None)
        assert len(parents) > 0
    return parents


def get_ancestors(lpad, fw_id):
    """return a list of all ancestors' fw_ids of a node with fw_id"""
    # better would be a direct pymongo query with a projection on links
    wfl_pl = lpad.get_wf_by_fw_id(fw_id).links.parent_links
    parents = wfl_pl.get(fw_id, [])
    ancestors = set()
    while parents:
        ancestors.update(parents)
        new_parents = set()
        for par in iter(parents):
            new_parents.update(wfl_pl.get(par, []))
        parents = new_parents
    return list(ancestors)
